// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';

class Event implements Identifiable<EventId> {
  final EventId id;

  Event(this.id);

  factory Event.fromJson(Room room, Map<String, dynamic> json) {

    var type = json['type'];

    // Skip if it's not a valid event id
    if(!EventId.isValidFullyQualified(json['event_id'])) {
      return null;
    }

    // These might be null
    var id = EventId(json['event_id']);
    var time = DateTime.fromMillisecondsSinceEpoch(json['origin_server_ts']);
    var sender = UserId(json['sender']);

    switch(type) {
      case 'm.room.name':
        var name = json['content']['name'];
        return NameChangeEvent(name, id, room, sender, time);
      default:
        return null;
    }
  }
}

class RoomEvent extends Event {
  final Room room;

  final UserId sender;
  final DateTime time;

  RoomEvent(EventId id, this.room, this.sender, this.time) : super(id);

}

class StateEvent extends RoomEvent {

  final String stateKey = '';

  StateEvent(EventId id, Room room, UserId sender, DateTime time)
      : super(id, room, sender, time);

}

class NameChangeEvent extends StateEvent {

  final String name;

  NameChangeEvent(this.name, EventId id, Room room, UserId sender, DateTime time)
      : super(id, room, sender, time);

}