// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/util/json.dart';

class Room implements Identifiable<RoomId>, JsonWritable {
  @override
  final RoomId id;

  Room(this.id, {String name}) : _name = name;

  String _name;
  String get name => _name;

  factory Room.fromJson(RoomId id, Map<String, dynamic> json) {
    final String name = json['name'];
    return Room(id, name: name);
  }

  // TODO: Generate
  dynamic toJson() => { 'name': name };

  void addEvents(List<dynamic> rawEvents) {
    for (var raw in rawEvents) {
      var event = Event.fromJson(this, raw);

      if (event is NameChangeEvent) {
        _name = event.name;
        print('Setting name to $name');
      }

    }
  }

}

class JoinedRoom extends Room {
  JoinedRoom(RoomId id, {String name}) : super(id, name: name);

  factory JoinedRoom.fromJson(RoomId id, Map<String, dynamic> json) {
    final String name = json['name'];
    return JoinedRoom(id, name: name);
  }
}